<?
/***************************************************************************
*                            Dolphin Smart Community Builder
*                              -------------------
*     begin                : Mon Mar 23 2006
*     copyright            : (C) 2007 BoonEx Group
*     website              : http://www.boonex.com
* This file is part of Dolphin - Smart Community Builder
*
* Dolphin is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the
* License, or  any later version.
*
* Dolphin is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
* You should have received a copy of the GNU General Public License along with Dolphin,
* see license.txt file; if not, write to marketing@boonex.com
***************************************************************************/

define('BX_FDB_STATUS_ACTIVE', 0);
define('BX_FDB_STATUS_APPROVAL', 1);
define('BX_FDB_STATUS_REJECTED', 2);

bx_import('BxDolModule');
bx_import('BxDolSubscription');
bx_import('BxDolAdminSettings');

require_once('BxFdbCmts.php');
require_once('BxFdbVoting.php');
require_once('BxFdbSearchResult.php');
require_once('BxFdbPrivacy.php');
require_once('BxFdbTextData.php');

/*
 * Feedback module by BoonEx
 *
 * This module is needed to manage user's feedback on the site.  
 *
 *
 * Profile's Wall:
 * no spy events
 *
 *
 *
 * Spy:
 * no spy events
 *
 *
 *
 * Memberships/ACL:
 * Doesn't depend on user's membership.
 *
 * 
 *
 * Service methods:
 *
 * Get post block. 
 * @see BxFdbModule::servicePostBlock
 * BxDolService::call('feedback', 'post_block');
 * @note is needed for internal usage.
 *
 * Get edit block.
 * @see BxFdbModule::serviceEditBlock
 * BxDolService::call('feedback', 'edit_block', array($mixed));
 * @note is needed for internal usage.
 *
 * Get administration block.
 * @see BxFdbModule::serviceAdminBlock
 * BxDolService::call('feedback', 'admin_block', array($iStart, $iPerPage, $sFilterValue));
 * @note is needed for internal usage.
 *
 * Get block with all feedback from current user.
 * @see BxFdbModule::serviceMyBlock
 * BxDolService::call('feedback', 'my_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 *
 * Get block with all feedback ordered by the time of posting.
 * @see BxFdbModule::serviceArchiveBlock
 * BxDolService::call('feedback', 'archive_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 *
 * Get the block with feedback content for view feedback page.
 * @see BxFdbModule::serviceViewBlock
 * BxDolService::call('feedback', 'view_block', array($sUri));
 * @note is needed for internal usage.
 *
 * Get the block with feedback comments for view feedback page.
 * @see BxFdbModule::serviceCommentBlock
 * BxDolService::call('feedback', 'comment_block', array($sUri));
 * @note is needed for internal usage.
 
 * Get the block with feedback votes for view feedback page.
 * @see BxFdbModule::serviceVoteBlock
 * BxDolService::call('feedback', 'vote_block', array($sUri));
 * @note is needed for internal usage.
 
 * Get the block with feedback actions for view feedback page.
 * @see BxFdbModule::serviceActionBlock
 * BxDolService::call('feedback', 'action_block', array($sUri));
 * @note is needed for internal usage. 
 *
 *
 * Alerts:
 * Alerts type/unit - 'feedback'
 * The following alerts are rised
 *
 *  post - new feedback is added
 *      $iObjectId - feedback id
 *      $iSenderId - feedback's owner
 *
 *  edit - feedback was modified
 *      $iObjectId - feedback id
 *      $iSenderId - feedback's owner
 *
 *  approve - feedback was approved
 *      $iObjectId - feedback id
 *      $iSenderId - admin's id
 *
 *  reject - feedback was rejected
 *      $iObjectId - feedback id
 *      $iSenderId - admin's id
 *
 *  delete - feedback was deleted
 *      $iObjectId - feedback id
 *      $iSenderId - admin's id
 *
 */
class BxFdbModule extends BxDolModule {
    var $_oPrivacy;
    var $_oTextData;    
        
	/**
	 * Constructor
	 */
	function BxFdbModule($aModule) {
	    parent::BxDolModule($aModule);	    
	    $this->_oConfig->init($this->_oDb);
	    $this->_oTemplate->init($this->_oDb);
	
	    $this->_oPrivacy = new BxFdbPrivacy($this);
	    $this->_oTextData = new BxFdbTextData($this);

		//--- Define Membership Actions ---//
        defineMembershipActions(array('feedback delete'), 'ACTION_ID_');	    
	}
	function getSearchResult() {
	    return new BxFdbSearchResult($this);
	} 


	/**
	 * 
	 * Admin Settings Methods
	 * 
	 */
	function getSettingsForm($mixedResult) {
	    $iId = (int)$this->_oDb->getOne("SELECT `ID` FROM `sys_options_cats` WHERE `name`='Feedback'");
	    if(empty($iId))
	       return MsgBox('_feedback_msg_no_results');
	       
        $oSettings = new BxDolAdminSettings($iId, BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . 'admin');
        $sResult = $oSettings->getForm();
        	       
        if($mixedResult !== true && !empty($mixedResult))
            $sResult = $mixedResult . $sResult;

        return $sResult;
	}
	function setSettings($aData) {
	    $iId = (int)$this->_oDb->getOne("SELECT `ID` FROM `sys_options_cats` WHERE `name`='Feedback'");
	    if(empty($iId))
	       return MsgBox(_t('_feedback_msg_no_results'));
	       
	    $oSettings = new BxDolAdminSettings($iId);
	    return $oSettings->saveChanges($_POST);
	}
	
	
	/**
	 * Service methods
	 */
	function servicePostBlock() {
	    if(!$this->isLogged())
            return MsgBox(_t('_feedback_msg_no_results'));

	    $iStatus = $this->_oConfig->isAutoapprove() ? BX_FDB_STATUS_ACTIVE : BX_FDB_STATUS_APPROVAL;
	    $aVariables = array(
	       'post_form' => $this->_oTextData->getPostForm(array('status' => $iStatus))
	    );
	    return $this->_oTemplate->parseHtmlByName('post.html', $aVariables);
	}
	function serviceEditBlock($mixed) {
	    if(!$this->isLogged())
            return MsgBox(_t('_feedback_msg_no_results'));

	    if(is_string($mixed))
	       $aFeedback = $this->_oDb->getFeedback(array('sample_type' => 'uri', 'uri' => $mixed));
	    else if(is_int($mixed))
	       $aFeedback = $this->_oDb->getFeedback(array('sample_type' => 'id', 'id' => $mixed));

	    $iStatus = $this->_oConfig->isAutoapprove() ? BX_FDB_STATUS_ACTIVE : BX_FDB_STATUS_APPROVAL;
	    $aVariables = array(
	       'post_form' => $this->_oTextData->getEditForm($aFeedback, array('status' => $iStatus))
	    );
	    return $this->_oTemplate->parseHtmlByName('post.html', $aVariables);
	}
	function serviceAdminBlock($iStart = 0, $iPerPage = 0, $sFilterValue = '') {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();
        
        return $this->_oTemplate->displayAdminBlock(array(
	       'sample_type' => 'admin', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage,
	       'admin_panel' => true,
	       'filter_value' => $sFilterValue,
	       'search_result_object' => $this->getSearchResult()
        ));
	}
	function serviceMyBlock($iStart = 0, $iPerPage = 0) {
	    if(!$this->isLogged())
            return MsgBox(_t('_feedback_msg_no_results'));

	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'owner', 
	       'viewer_id' => $this->_oTextData->getAuthorId(),
	       'viewer_type' => $this->_oTextData->getViewerType(), 	       
	       'start' => $iStart, 
	       'count' => $iPerPage
        ));
	}
	function serviceArchiveBlock($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getIndexNumber();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'archive', 
	       'viewer_id' => $this->_oTextData->getAuthorId(),
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage,
	       'show_empty' => $bShowEmpty
        ));
	}
	
	/**
	 * View Feedback section.
	 */
	function serviceViewBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aFeedback = $this->_oDb->getFeedback($aParams);
	    
	    if(empty($aFeedback))
            BxDolTemplate::displayPageNotFound();
	    
        $GLOBALS['oTopMenu']->setCustomSubHeader($aFeedback['caption']);
        $GLOBALS['oTopMenu']->setCustomBreadcrumbs(array(
            _t('_feedback_top_menu_sitem') => BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . 'index/',
            $aFeedback['caption'] => '')
        );
        
	    return $this->_oTemplate->displayList(array(
	       'sample_type' => 'view', 
	       'viewer_id' => $this->_oTextData->getAuthorId(),
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'uri' => $aFeedback['uri']
        ));
	}
	function serviceCommentBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aFeedback = $this->_oDb->getFeedback($aParams);
	    if(!$this->_oConfig->isCommentsAllowed() || !$this->_oPrivacy->check('comment', $aFeedback['id'], $this->_oTextData->getAuthorId())) 
	       return '';
	       
        if($aFeedback['status'] != BX_FDB_STATUS_ACTIVE)
            return MsgBox(_t('_feedback_msg_no_results'));

        $this->_oTemplate->addCss(array('cmts.css'));
        $oComments = new BxFdbCmts($this->_oConfig->getCommentsSystemName(), $aFeedback['id']);
	    return $oComments->getCommentsFirst('comment');
	}
	function serviceVoteBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aFeedback = $this->_oDb->getFeedback($aParams);
	    if(!$this->_oConfig->isVotesAllowed()  || !$this->_oPrivacy->check('vote', $aFeedback['id'], $this->_oTextData->getAuthorId()))
            return '';

        if($aFeedback['status'] != BX_FDB_STATUS_ACTIVE)
            return MsgBox(_t('_feedback_msg_no_results'));

        $oVotes = new BxFdbVoting($this->_oConfig->getVotesSystemName(), $aFeedback['id']);
	    return $oVotes->getBigVoting();
	}
	function serviceActionBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aFeedback = $this->_oDb->getFeedback($aParams);

	    if($aFeedback['status'] != BX_FDB_STATUS_ACTIVE)
            return MsgBox(_t('_feedback_msg_no_results'));

        $oSubscription = new BxDolSubscription();
        $aButton = $oSubscription->getButton($this->getUserId(), 'bx_' . $this->_oConfig->getUri(), '', $aFeedback['id']);
        
        $aReplacement['sbs_feedback_title'] = $aButton['title']; 
        $aReplacement['sbs_feedback_script'] = $aButton['script'];
                
        if($this->_isDeleteAllowed($aFeedback['author_id'])) {
        	$this->_oTemplate->addJsTranslation(array('_feedback_msg_success_delete', '_feedback_msg_failed_delete'));
        	
        	$aReplacement['del_feedback_title'] = _t('_feedback_actions_delete');
        	$aReplacement['del_feedback_script'] = $this->_oConfig->getJsObject() . '.deleteFeedback(' . $aFeedback['id'] . ')';
        }
        else
        	$aReplacement['del_feedback_title'] = '';
        	
		return $oSubscription->getData() . $GLOBALS['oFunctions']->genObjectsActions($aReplacement, 'bx_' . $this->_oConfig->getUri());
	}
	function serviceGetSubscriptionParams($sUnit, $sAction, $iObjectId) {
		$sUnit = str_replace('bx_', '_', $sUnit);
		if(empty($sAction))
			$sAction = 'main';

		$aItem = $this->_oDb->getFeedback(array('sample_type' => 'id', 'id' => $iObjectId));

		return array(
			'template' => array(
				'Subscription' => _t($sUnit . '_sbs_' . $sAction, $aItem['caption']), 
				'ViewLink' => BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri()  . 'view/' . $aItem['uri']
			)
		);
	}

	/**
	 * Action methods	 
	 */
	function actionGetFeedback($sSampleType = 'all', $iStart = 0, $iPerPage = 0) {
	    check_logged();
	    
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayList(array(
            'sample_type' => $sSampleType, 
            'viewer_type' => $this->_oTextData->getViewerType(), 
            'start' => $iStart, 
            'count' => $iPerPage
        ));
	}
	function actionApprove() {
		$iId = (int)$_POST['id'];

		return !empty($iId) ? $this->_actApprove($iId) : false;
	}
	function actionDelete() {
		$iId = (int)$_POST['id'];

		return !empty($iId) ? $this->_actDelete($iId) : false;
	}
	
	function _actApprove($aIds, $bPositive = true) {
	    if(!isAdmin()) 
            return false;

		if(is_int($aIds) || is_string($aIds))
			$aIds = array((int)$aIds);

	    $bResult = $this->_oDb->updateFeedback($aIds, array('status' => ($bPositive ? BX_FDB_STATUS_ACTIVE : BX_FDB_STATUS_REJECTED)));
	    if($bResult)
	        foreach($aIds as $iId) {
    	        //--- Feedback -> Approve/Reject for Alerts Engine ---//
                bx_import('BxDolAlerts');
                $oAlert = new BxDolAlerts('bx_' . $this->_oConfig->getUri(), ($bPositive ? 'approve' : 'reject'), $iId, BxDolTextData::getAuthorId());
                $oAlert->alert();
                //--- Feedback -> Approve/Reject for Alerts Engine ---//
                
                //--- Reparse Global Tags ---//
                bx_import('BxDolTags');
                $oTags = new BxDolTags();
                $oTags->reparseObjTags('bx_feedback', $iId);
                //--- Reparse Global Tags ---//
	        }
        
	    return $bResult;
	}
	function _actDelete($aIds) {
		if(is_int($aIds) || is_string($aIds)) {
	    	$aFeedback = $this->_oDb->getFeedback(array('sample_type' => 'id', 'id' => (int)$aIds));
			if(!$this->_isDeleteAllowed($aFeedback['author_id'], true)) 
            	return false;

			$aIds = array((int)$aIds);
		}
		else if(!$this->_isDeleteAllowed(0, true)) 
			return false;

	    $bResult = $this->_oDb->deleteFeedback($aIds);
	    if($bResult)
	        foreach($aIds as $iId) {
    	        //--- Feedback -> Delete for Alerts Engine ---//
                bx_import('BxDolAlerts');
                $oAlert = new BxDolAlerts('bx_' . $this->_oConfig->getUri(), 'delete', $iId, BxDolTextData::getAuthorId());
                $oAlert->alert();
                //--- Feedback -> Delete for Alerts Engine ---//
                
                //--- Reparse Global Tags ---//
                bx_import('BxDolTags');
                $oTags = new BxDolTags();
                $oTags->reparseObjTags('bx_feedback', $iId);
                //--- Reparse Global Tags ---//
	        }

	    return $bResult;
	}
	function _isDeleteAllowed($iAuthorId = 0, $bPerform = false) {
		if(!isLogged())
			return false;

		if(isAdmin())
			return true;

		$iUserId = getLoggedId();
		if($iAuthorId != 0 && $iAuthorId == $iUserId)
			return true;
		
		$aCheckResult = checkAction($iUserId, ACTION_ID_FEEDBACK_DELETE, $bPerform);			
		return $aCheckResult[CHECK_ACTION_RESULT] == CHECK_ACTION_RESULT_ALLOWED;
	}
}
?>