<?
/***************************************************************************
*                            Dolphin Smart Community Builder
*                              -------------------
*     begin                : Mon Mar 23 2006
*     copyright            : (C) 2007 BoonEx Group
*     website              : http://www.boonex.com
* This file is part of Dolphin - Smart Community Builder
*
* Dolphin is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the
* License, or  any later version.
*
* Dolphin is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
* You should have received a copy of the GNU General Public License along with Dolphin,
* see license.txt file; if not, write to marketing@boonex.com
***************************************************************************/

bx_import('BxDolViews');
bx_import('BxDolAlerts');
bx_import('BxDolModule');
bx_import('BxDolPaginate');
bx_import('BxDolCategories');
bx_import('BxDolRssFactory');
bx_import('BxDolSubscription');
bx_import('BxDolAdminSettings');

require_once('BxNewsCalendar.php');
require_once('BxNewsCmts.php');
require_once('BxNewsVoting.php');
require_once('BxNewsSearchResult.php');
require_once('BxNewsTextData.php');

/*
 * News module by BoonEx
 *
 * This module is needed to manage site news.
 *
 *
 * Profile's Wall:
 * no spy events
 *
 *
 *
 * Spy:
 * no spy events
 *
 *
 *
 * Memberships/ACL:
 * Doesn't depend on user's membership.
 *
 * 
 *
 * Service methods:
 *
 * Get post block. 
 * @see BxNewsModule::servicePostBlock
 * BxDolService::call('news', 'post_block');
 * @note is needed for internal usage.
 *
 * Get edit block.
 * @see BxNewsModule::serviceEditBlock
 * BxDolService::call('news', 'edit_block', array($mixed));
 * @note is needed for internal usage.
 *
 * Get administration block.
 * @see BxNewsModule::serviceAdminBlock
 * BxDolService::call('news', 'admin_block', array($iStart, $iPerPage, $sFilterValue));
 * @note is needed for internal usage.
 *
 * Get block with all news ordered by the time of posting.
 * @see BxNewsModule::serviceArchiveBlock
 * BxDolService::call('news', 'archive_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 *
 * Get block with news marked as featured.
 * @see BxNewsModule::serviceFeaturedBlock
 * BxDolService::call('news', 'featured_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 *
 * Get block with news ordered by their rating.
 * @see BxNewsModule::serviceTopRatedBlock
 * BxDolService::call('news', 'top_rated_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 *
 * Get block with all news ordered by their popularity(number of views).
 * @see BxNewsModule::servicePopularBlock
 * BxDolService::call('news', 'popular_block', array($iStart, $iPerPage));
 * @note is needed for internal usage.
 * 
 * Get the block with news content for view news page.
 * @see BxNewsModule::serviceViewBlock
 * BxDolService::call('news', 'view_block', array($sUri));
 * @note is needed for internal usage.
 *
 * Get the block with news comments for view news page.
 * @see BxNewsModule::serviceCommentBlock
 * BxDolService::call('news', 'comment_block', array($sUri));
 * @note is needed for internal usage.
 
 * Get the block with news votes for view news page.
 * @see BxNewsModule::serviceVoteBlock
 * BxDolService::call('news', 'vote_block', array($sUri));
 * @note is needed for internal usage.
 *
 * Get the block with news actions for view news page.
 * @see BxNewsModule::serviceActionBlock
 * BxDolService::call('news', 'action_block', array($sUri));
 * @note is needed for internal usage. 
 *
 *
 * Alerts:
 * Alerts type/unit - 'news'
 * The following alerts are rised
 *
 *  post - news is added
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 *  edit - news was modified
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 *  featured - news was marked as featured
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 *  publish - news was published
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 *  unpublish - news was unpublished
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 *  delete - news was deleted
 *      $iObjectId - news id
 *      $iSenderId - admin's id
 *
 */
class BxNewsModule extends BxDolModule {
    var $_oTextData;
    
	/**
	 * Constructor
	 */
	function BxNewsModule($aModule) {
	    parent::BxDolModule($aModule);
	    $this->_oConfig->init($this->_oDb);
	    $this->_oTemplate->init($this->_oDb);
	    	    
	    $this->_oTextData = new BxNewsTextData($this);
	    
		//--- Define Membership Actions ---//
        defineMembershipActions(array('news delete'), 'ACTION_ID_');
	}
		
	function getCalendar($iYear = 0, $iMonth = 0) {
	    return new BxNewsCalendar($iYear, $iMonth, $this->_oDb, $this->_oConfig);
	}
	function getSearchResult() {
	    return new BxNewsSearchResult($this);
	}
	function getSearchContent($sType, $sValue, $iPage = 1, $iPerPage = 0) {
		$iPerPage = !empty($iPerPage) ? $iPerPage : $this->_oConfig->getPerPage();
 
	    $oSearchResult = new BxNewsSearchResult($this);		    
	    $oSearchResult->aCurrent['restriction'][$sType]['value'] = $sValue;
	    $oSearchResult->aCurrent['paginate']['forcePage'] = $iPage;
	    $oSearchResult->aCurrent['paginate']['perPage'] = $iPerPage;
	    $sCode = $oSearchResult->displayResultBlock();

	    if(!empty($sCode)) {
	        $oPaginate = new BxDolPaginate(array(
	            'page_url' => BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . $sType . '/' . $sValue . '/{page}/{per_page}',
	            'count' => $oSearchResult->aCurrent['paginate']['totalNum'],
	            'per_page' => $iPerPage,
	            'page' => $iPage,
	            'per_page_changer' => true,
	            'page_reloader' => false,
	            'on_change_page' => '',
	            'on_change_per_page' => '',
	        ));
	        $sCode .= $oPaginate->getPaginate();
	    }

	    return $sCode;
	}
	
	/**
	 * 
	 * Admin Settings Methods
	 * 
	 */
	function getSettingsForm($mixedResult) {
	    $iId = (int)$this->_oDb->getOne("SELECT `ID` FROM `sys_options_cats` WHERE `name`='News'");
	    if(empty($iId))
	       return MsgBox('_news_msg_no_results');
	       
        $oSettings = new BxDolAdminSettings($iId, BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . 'admin');
        $sResult = $oSettings->getForm();
        	       
        if($mixedResult !== true && !empty($mixedResult))
            $sResult = $mixedResult . $sResult;

        return $sResult;
	}
	function setSettings($aData) {
	    $iId = (int)$this->_oDb->getOne("SELECT `ID` FROM `sys_options_cats` WHERE `name`='News'");
	    if(empty($iId))
	       return MsgBox(_t('_news_msg_no_results'));
	       
	    $oSettings = new BxDolAdminSettings($iId);
	    return $oSettings->saveChanges($_POST);
	}
	
	
	/**
	 * Service methods
	 */
	function serviceNewsRss($iLength = 0) {
	    return $this->actionRss($iLength);
	}
	function servicePostBlock() {
	    
	    $aVariables = array(
	       'include_css' => $this->_oTemplate->addCss(array('post.css'), true),
	       'post_form' => $this->_oTextData->getPostForm()
	    );
	    return $this->_oTemplate->parseHtmlByName('post.html', $aVariables);
	}
	function serviceEditBlock($mixed) {
	    if(is_string($mixed))
	       $aNews = $this->_oDb->getNews(array('sample_type' => 'uri', 'uri' => $mixed));
	    else if(is_int($mixed))
	       $aNews = $this->_oDb->getNews(array('sample_type' => 'id', 'id' => $mixed));

	    $aVariables = array(
	       'include_css' => $this->_oTemplate->addCss(array('post.css'), true),
	       'post_form' => $this->_oTextData->getEditForm($aNews)
	    );
	    return $this->_oTemplate->parseHtmlByName('post.html', $aVariables);
	}
	/**
	 * Replace with a number of separate functions
	 */
	function serviceAdminBlock($iStart = 0, $iPerPage = 0, $sFilterValue = '') {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();
        
        return $this->_oTemplate->displayAdminBlock(array(
	       'sample_type' => 'admin', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage,
	       'admin_panel' => true,
	       'filter_value' => $sFilterValue,
	       'search_result_object' => $this->getSearchResult()
        ));
	}
	function serviceArchiveBlockIndex($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
		if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getIndexNumber();

		return $this->serviceArchiveBlock($iStart, $iPerPage, $bShowEmpty);
	}
	function serviceArchiveBlockMember($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
		if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getMemberNumber();

		return $this->serviceArchiveBlock($iStart, $iPerPage, $bShowEmpty);
	}
	function serviceArchiveBlock($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'archive', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage,
	       'show_empty' => $bShowEmpty
        ));
	}
	function serviceFeaturedBlockIndex($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
		if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getIndexNumber();

		return $this->serviceFeaturedBlock($iStart, $iPerPage, $bShowEmpty);
	}
	function serviceFeaturedBlockMember($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
		if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getMemberNumber();

		return $this->serviceFeaturedBlock($iStart, $iPerPage, $bShowEmpty);
	}
	function serviceFeaturedBlock($iStart = 0, $iPerPage = 0, $bShowEmpty = true) {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'featured', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage,
	       'show_empty' => $bShowEmpty
        ));
	}
	function serviceTopRatedBlock($iStart = 0, $iPerPage = 0) {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'top_rated', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage
        ));
	}
	function servicePopularBlock($iStart = 0, $iPerPage = 0) {
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayBlock(array(
	       'sample_type' => 'popular', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'start' => $iStart, 
	       'count' => $iPerPage
        ));
	}
	
	
	
	/**
	 * View Single News Blocks
	 */
	function serviceViewBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aNews = $this->_oDb->getNews($aParams);
	    
	    $oView = new BxDolViews('bx_' . $this->_oConfig->getUri(), $aNews['id']);
        $oView->makeView();
        
        $this->_oTemplate->setPageTitle($aNews['caption']);
        $GLOBALS['oTopMenu']->setCustomSubHeader($aNews['caption']);
        $GLOBALS['oTopMenu']->setCustomBreadcrumbs(array(
            _t('_news_top_menu_item') => BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . 'index/',
            $aNews['caption'] => '')
        );
	    return $this->_oTemplate->displayList(array(
	       'sample_type' => 'view', 
	       'viewer_type' => $this->_oTextData->getViewerType(), 
	       'uri' => $aNews['uri']
        ));
	}
	function serviceCommentBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aNews = $this->_oDb->getNews($aParams);
	    if(!$this->_oConfig->isCommentsAllowed()) 
	       return '';

	    if($aNews['status'] != BX_TD_STATUS_ACTIVE)
            return MsgBox(_t('_news_msg_no_results'));

        $this->_oTemplate->addCss(array('cmts.css'));
        $oComments = new BxNewsCmts($this->_oConfig->getCommentsSystemName(), $aNews['id']);
	    return $oComments->getCommentsFirst('comment');
	}
	function serviceVoteBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aNews = $this->_oDb->getNews($aParams);
	    if(!$this->_oConfig->isVotesAllowed()) 
	       return '';
	       
	    if($aNews['status'] != BX_TD_STATUS_ACTIVE)
            return MsgBox(_t('_news_msg_no_results'));

        $oVotes = new BxNewsVoting($this->_oConfig->getVotesSystemName(), $aNews['id']);
	    return $oVotes->getBigVoting();
	}
	function serviceActionBlock($sUri) {
	    $aParams = is_numeric($sUri) ? array('sample_type' => 'id', 'id' => $sUri) : array('sample_type' => 'uri', 'uri' => $sUri);
	    $aNews = $this->_oDb->getNews($aParams);
	    
	    if($aNews['status'] != BX_TD_STATUS_ACTIVE)
            return MsgBox(_t('_news_msg_no_results'));

        $oSubscription = new BxDolSubscription();
        $aButton = $oSubscription->getButton($this->getUserId(), 'bx_' . $this->_oConfig->getUri(), '', $aNews['id']);        

		$aReplacement['sbs_news_title'] = $aButton['title']; 
        $aReplacement['sbs_news_script'] = $aButton['script'];
                
        if($this->_isDeleteAllowed()) {
        	$this->_oTemplate->addJsTranslation(array('_news_msg_success_delete', '_news_msg_failed_delete'));
        	
        	$aReplacement['del_news_title'] = _t('_news_actions_delete');
        	$aReplacement['del_news_script'] = $this->_oConfig->getJsObject() . '.deleteNews(' . $aNews['id'] . ')';
        }
        else
        	$aReplacement['del_news_title'] = '';
        	
		return $oSubscription->getData() . $GLOBALS['oFunctions']->genObjectsActions($aReplacement, 'bx_' . $this->_oConfig->getUri());
	}
	function serviceGetSubscriptionParams($sUnit, $sAction, $iObjectId) {
		$sUnit = str_replace('bx_', '_', $sUnit);
		if(empty($sAction))
			$sAction = 'main';

		$aItem = $this->_oDb->getNews(array('sample_type' => 'id', 'id' => $iObjectId));

		return array(
			'template' => array(
				'Subscription' => _t($sUnit . '_sbs_' . $sAction, $aItem['caption']), 
				'ViewLink' => BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri()  . 'view/' . $aItem['uri']
			)
		);
	}

	/**
	 * Action methods
	 *
	 * Get RSS for news.
	 *
	 * @return string with RSS.
	 */
	function actionRss($iLength = 0) {
		$iLength = $iLength != 0 ? $iLength : (int)$this->_oConfig->getRssLength();

	    $aEntries = $this->_oDb->getNews(array(
            'sample_type' => 'archive', 
            'viewer_type' => $this->_oTextData->getViewerType(), 
            'start' => 0, 
            'count' => $iLength
        ));

        $sRssBaseUrl = BX_DOL_URL_ROOT . $this->_oConfig->getBaseUri() . 'view/';
        $aRssData = array();
        foreach($aEntries as $aEntry) {
            if(empty($aEntry['caption'])) continue;

        	$aRssData[$aEntry['id']] = array(
        	   'UnitID' => $aEntry['id'],
        	   'OwnerID' => $aEntry['id'],
        	   'UnitTitle' => $aEntry['caption'],
        	   'UnitLink' => $sRssBaseUrl . $aEntry['uri'],
        	   'UnitDesc' => $aEntry['content'],
        	   'UnitDateTimeUTS' => $aEntry['date'],
        	   'UnitIcon' => ''
            );
        }

	    $oRss = new BxDolRssFactory();
	    return $oRss->GenRssByData($aRssData, _t('_news_rss_caption'), $sRssBaseUrl);
	}
	function actionGetNews($sSampleType = 'all', $iStart = 0, $iPerPage = 0) {
	    check_logged();
	    
	    if(empty($iPerPage))
            $iPerPage = $this->_oConfig->getPerPage();

	    return $this->_oTemplate->displayList(array(
            'sample_type' => $sSampleType, 
            'viewer_type' => $this->_oTextData->getViewerType(), 
            'start' => $iStart, 
            'count' => $iPerPage
        ));
	}
	function actionFeatured() {
		$iId = (int)$_POST['id'];

		return !empty($iId) ? $this->_actFeatured($iId) : false;
	}
	function actionPublish() {
		$iId = (int)$_POST['id'];

		return !empty($iId) ? $this->_actPublish($iId) : false;
	}
	function actionDelete() {
		$iId = (int)$_POST['id'];

		return !empty($iId) ? $this->_actDelete($iId) : false;
	}

	function _actFeatured($aIds, $bPositive = true) {	    
	    if(!isAdmin()) 
            return false;

		if(is_int($aIds) || is_string($aIds))
			$aIds = array((int)$aIds);

	    $bResult = $this->_oDb->updateNews($aIds, array('featured' => ($bPositive ? 1 : 0)));
	    if($bResult)
	        foreach($aIds as $iId) {
    	        //--- News -> Featured for Alerts Engine ---//
                bx_import('BxDolAlerts');
                $oAlert = new BxDolAlerts('bx_' . $this->_oConfig->getUri(), 'featured', $iId, BxDolTextData::getAuthorId());
                $oAlert->alert();
                //--- News -> Featured for Alerts Engine ---//
	        }

	    return $bResult;
	}
	function _actPublish($aIds, $bPositive = true) {
	    if(!isAdmin()) 
            return false;

		if(is_int($aIds) || is_string($aIds))
			$aIds = array((int)$aIds);

	    $bResult = $this->_oDb->updateNews($aIds, array('status' => ($bPositive ? BX_TD_STATUS_ACTIVE : BX_TD_STATUS_INACTIVE)));	    
	    if($bResult)
	        foreach($aIds as $iId) {
    	        //--- News -> Publish/Unpublish for Alerts Engine ---//
                $oAlert = new BxDolAlerts('bx_' . $this->_oConfig->getUri(), ($bPositive ? 'publish' : 'unpublish'), $iId, BxDolTextData::getAuthorId());
                $oAlert->alert();
                //--- News -> Publish/Unpublish for Alerts Engine ---//
                
                //--- Reparse Global Tags ---//
                $oTags = new BxDolTags();
                $oTags->reparseObjTags('bx_' . $this->_oConfig->getUri(), $iId);
                //--- Reparse Global Tags ---//
                
                //--- Reparse Global Categories ---//            
                $oCategories = new BxDolCategories();
                $oCategories->reparseObjTags('bx_' . $this->_oConfig->getUri(), $iId);
                //--- Reparse Global Categories ---//
	        }
        
	    return $bResult;
	}
	function _actDelete($aIds) {	    
	    if(!$this->_isDeleteAllowed(true)) 
            return false;
            
		if(is_int($aIds) || is_string($aIds))
			$aIds = array((int)$aIds);
	    	    
	    $bResult = $this->_oDb->deleteNews($aIds);
	    if($bResult) {
	        $oTags = new BxDolTags();
            $oCategories = new BxDolCategories();
            $oSubscription = new BxDolSubscription();
            
	        foreach($aIds as $iId) {
    	        //--- News -> Delete for Alerts Engine ---//
                $oAlert = new BxDolAlerts('bx_' . $this->_oConfig->getUri(), 'delete', $iId, BxDolTextData::getAuthorId());
                $oAlert->alert();
                //--- News -> Delete for Alerts Engine ---//
                
                //--- Reparse Global Tags ---//                                
                $oTags->reparseObjTags('bx_' . $this->_oConfig->getUri(), $iId);
                //--- Reparse Global Tags ---//
                
                //--- Reparse Global Categories ---//                            
                $oCategories->reparseObjTags('bx_' . $this->_oConfig->getUri(), $iId);
                //--- Reparse Global Categories ---//
                
                //--- Remove all subscriptions ---//
                $oSubscription->unsubscribe(array('type' => 'object_id', 'unit' => 'bx_' . $this->_oConfig->getUri(), 'object_id' => $iId));
                //--- Remove all subscriptions ---//
	        }
	    }
	    return $bResult;
	}
	function _isDeleteAllowed($bPerform = false) {
		if(!isLogged())
			return false;

		if(isAdmin())
			return true;
			
		$aCheckResult = checkAction(getLoggedId(), ACTION_ID_NEWS_DELETE, $bPerform);			
		return $aCheckResult[CHECK_ACTION_RESULT] == CHECK_ACTION_RESULT_ALLOWED;
	}
}
?>