<?
/***************************************************************************
*                            Dolphin Smart Community Builder
*                              -------------------
*     begin                : Mon Mar 23 2006
*     copyright            : (C) 2007 BoonEx Group
*     website              : http://www.boonex.com
* This file is part of Dolphin - Smart Community Builder
*
* Dolphin is free software; you can redistribute it and/or modify it under
* the terms of the GNU General Public License as published by the
* Free Software Foundation; either version 2 of the
* License, or  any later version.
*
* Dolphin is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
* You should have received a copy of the GNU General Public License along with Dolphin,
* see license.txt file; if not, write to marketing@boonex.com
***************************************************************************/

define ('BX_PROFILER_DISPLAY', 1);

require_once(BX_DIRECTORY_PATH_MODULES . 'boonex/profiler/install/config.php');

$GLOBALS['bx_profiler_module'] = array (
    'title' => $aConfig['title'],
    'vendor' => $aConfig['vendor'],
    'path' => $aConfig['home_dir'],
    'uri' => $aConfig['home_uri'],
    'class_prefix' => $aConfig['class_prefix'],
    'db_prefix' => $aConfig['db_prefix'],
);

bx_import('Template', $GLOBALS['bx_profiler_module']);
bx_import('Config', $GLOBALS['bx_profiler_module']);

class BxProfiler {

    var $oConfig, $oTemplate;

    var $aConf = array ();

    var $_iTimeStart = 0;

    var $_aQueries = array();
    var $_sQueryIndex = 0;

    var $_aModules = array();
    var $_aModulesNames = array();
    var $_aModulesLevel = 0;

    var $_sLogDateFormat = "Y-m-d H:i:s";
    var $_sLogMaxArgLength = 64;
    var $_sLogFilename = 64;

	/**
	 * constructor
	 */
    function BxProfiler($iTimeStart) {

        $this->oConfig = new BxProfilerConfig ($GLOBALS['bx_profiler_module']);
        $this->oTemplate = new BxProfilerTemplate ($this->oConfig);

        $aCss = array (
            BX_DOL_URL_MODULES.'boonex/profiler/templates/base/css/profiler.css',
            BX_DOL_URL_MODULES.'boonex/profiler/plugins/jush/jush.css',
        );
        $aJs = array (
            BX_DOL_URL_MODULES.'boonex/profiler/js/profiler.js',
            BX_DOL_URL_ROOT.'plugins/jquery/jquery.tablesorter.js',
            BX_DOL_URL_MODULES.'boonex/profiler/plugins/jush/jush.js',
        );

        foreach ($aCss as $sCssUrl) {
            $this->oTemplate->addCss ($sCssUrl);
            $this->oTemplate->addCssAdmin ($sCssUrl);
        }
        foreach ($aJs as $sJsUrl) {
            $this->oTemplate->addJs($sJsUrl);
            $this->oTemplate->addJsAdmin($sJsUrl);
        }

        if (getParam ('bx_profiler_long_sql_queries_log'))
            $this->aConf['long_query'] = getParam ('bx_profiler_long_sql_queries_time');
        if (getParam ('bx_profiler_long_module_query_log'))
            $this->aConf['long_module'] = getParam ('bx_profiler_long_module_query_time');
        if (getParam ('bx_profiler_long_page_log'))
            $this->aConf['long_page'] = getParam ('bx_profiler_long_page_time');

        $this->_iTimeStart = $iTimeStart;        
    }

    /**
     * output profiler info
     */
    function output () {

        $iPageTIme = $this->_getCurrentDelay ();
        if (isset($this->aConf['long_page']) && $iPageTIme > $this->aConf['long_page'])
            $this->logPageOpen ($iPageTIme);

        switch (getParam('bx_profiler_show_debug_panel')) {
        case 'all':
            break;
        case 'admins':
            if (!$GLOBALS['logged']['admin'])
                return;
            break;
        case 'none':
        default:
            return;
        }
        echo $this->_plankMain ();
        echo $this->_plankSql ();
        echo $this->_plankModules ();
    } 

    function _logBegin ($s) {
        $sDate = date ($this->_sLogDateFormat);
        return "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n" . $sDate . " " . $s . "\n" .
        "User ID: " . $_COOKIE['memberID']  . "\n" .
        "User role: " . ($GLOBALS['logged']['admin'] ? 'admin' : ($GLOBALS['logged']['member'] ? 'member' : 'guest')) . "\n";        
    }

    function _logEnd () {
        return "\n";
    }

    function _appendToLog ($s) {
        $f = fopen ( BX_DIRECTORY_PATH_MODULES . $GLOBALS['bx_profiler_module']['path'] . 'log/profiler.log', 'a');
        if (!$f)
            return;
        fwrite($f, $s);
        fclose($f);
    }

    function logSqlQuery ($iTime, $aSqlQuery, &$res) {
        $s  = $this->_logBegin ('LONG SQL QUERY: ' . $aSqlQuery['time']);
        $s .= "Rows: " . $aSqlQuery['rows'] . "\n";
        $s .= "Affected: " . $aSqlQuery['affected'] . "\n";
        $s .= "Query: " . $aSqlQuery['sql'] . "\n";
        if (getParam('bx_profiler_long_sql_queries_debug'))
            $s .= "Backtrace: \n" . $this->_debugBackTrace (3);
        $s .= $this->_logEnd();
        $this->_appendToLog ($s);
    }

    function logModuleQuery ($iTime, $aModuleQuery) {
        $s  = $this->_logBegin ('LONG MODULE QUERY: ' . $aModuleQuery['time']);
        $s .= "Module name: " . $aModuleQuery['name'] . "\n";
        $s .= "Query type: " . $aModuleQuery['type'] . "\n";
        $s .= "Class/file: " . $aModuleQuery['class/file'] . "\n";
        $s .= "Method: " . $aModuleQuery['method'] . "\n";
        if (getParam('bx_profiler_long_module_query_debug'))
            $s .= "Backtrace: \n" . $this->_debugBackTrace (3);
        $s .= $this->_logEnd();
        $this->_appendToLog ($s);
    }

    function logPageOpen ($iTime) {
        $s  = $this->_logBegin ('LONG PAGE OPEN: ' . $this->_formatTime($iTime, 5));
        $s .= "Request method: " . $_SERVER['REQUEST_METHOD'] . "\n";
        $s .= "Query string: " . $_SERVER['QUERY_STRING'] . "\n";
        $s .= "Request URI: " . $_SERVER['REQUEST_URI'] . "\n";
        $s .= "Script name: " . $_SERVER['SCRIPT_NAME'] . "\n";
        $s .= "PHP self: " . $_SERVER['PHP_SELF'] . "\n";
        if (getParam('bx_profiler_long_page_debug'))
            $s .= "All server vars: \n" . print_r ($_SERVER, true);
        $s .= $this->_logEnd();
        $this->_appendToLog ($s);
    }

    function beginModule($sType, $sHash, &$aModule, $sClassFile, $sMethod = '' ) {
        ++$this->_aModulesLevel;
        $this->_aModulesNames[$aModule['title']] = isset($this->_aModulesNames[$aModule['title']]) ? $this->_aModulesNames[$aModule['title']] + 1 : 1;
        $this->_aModules[$sHash] = array (
            'name' => str_repeat('&#160;&#160;', $this->_aModulesLevel-1) . $aModule['title'],
            'type' => $sType,
            'class/file' => $sClassFile,
            'method' => $sMethod,
            'begin' => microtime (),
            'time' => -1,
        );
    }

    function endModule($sType, $sHash) {
        --$this->_aModulesLevel;
        $iTime = $this->_calcTime ($this->_aModules[$sHash]['begin']);
        unset ($this->_aModules[$sHash]['begin']);
        $this->_aModules[$sHash]['time'] = $this->_formatTime($iTime, 5);
        if (isset($this->aConf['long_module']) && $iTime > $this->aConf['long_module'])
            $this->logModuleQuery ($iTime, $this->_aModules[$sHash]);
    }

    function beginQuery ($sSql) {                
        $this->_sQueryIndex = md5 ($sSql.time().rand());
        $this->_aQueries[$this->_sQueryIndex]['sql'] = $sSql;
        $this->_aQueries[$this->_sQueryIndex]['begin'] = microtime ();        
    }

    function endQuery (&$res) {
        if (!$this->_sQueryIndex) 
            return;
        $iTime = $this->_calcTime ($this->_aQueries[$this->_sQueryIndex]['begin']);
        unset ($this->_aQueries[$this->_sQueryIndex]['begin']);
        $this->_aQueries[$this->_sQueryIndex]['time'] = $this->_formatTime($iTime, 5);
        $this->_aQueries[$this->_sQueryIndex]['rows'] = $res ? $GLOBALS['MySQL']->getNumRows($res) : '';
        $this->_aQueries[$this->_sQueryIndex]['affected'] = $res ? $GLOBALS['MySQL']->getAffectedRows($res) : '';
        if (isset($this->aConf['long_query']) && $iTime > $this->aConf['long_query'])
            $this->logSqlQuery ($iTime, $this->_aQueries[$this->_sQueryIndex], $res);
    }

    function _getCurrentDelay () {
        $i1 = explode(' ', microtime ());
        $i2 = explode(' ', $this->_iTimeStart);        
        return ($i1[0]+$i1[1]) - ($i2[0]+$i2[1]);
    }

    function _plankMain () {

        $sTime = $this->_formatTime($this->_getCurrentDelay ());
        if (function_exists('memory_get_usage'))
            $sMemory = $this->_formatBytes(memory_get_usage(true)) . ' of ' . ini_get('memory_limit') . ' allowed';

        return $this->oTemplate->plank(
            $this->oTemplate->nameValue('Time:', $sTime) .
            (function_exists('memory_get_usage') ? $this->oTemplate->nameValue('Memory:', $sMemory) : '') .
            $this->oTemplate->nameValue('PHP:', phpversion()) . 
            $this->oTemplate->nameValue('SAPI:', php_sapi_name()) . 
            $this->oTemplate->nameValue('OS:', php_uname('s r m')) 
        ); 
    }

    function _plankSql () {

        if (!$GLOBALS['bx_profiler']->_aQueries)
            return;

        $sQueries = count($GLOBALS['bx_profiler']->_aQueries) . ' queries'; 

        return $this->oTemplate->plank(
            $this->oTemplate->nameValue('SQL:', $sQueries),
            $this->oTemplate->table($GLOBALS['bx_profiler']->_aQueries, 'sql')
        );
    }

    function _plankModules () {

        if (!$GLOBALS['bx_profiler']->_aModules)
            return;

        $sModules = count($GLOBALS['bx_profiler']->_aModulesNames) . ' modules loaded';
        $sModulesQueries = count($GLOBALS['bx_profiler']->_aModules) . ' modules queries';

        return $this->oTemplate->plank(
            $this->oTemplate->nameValue('Modules:', $sModules) . 
            $this->oTemplate->nameValue('Modules Queries:', $sModulesQueries),
            $this->oTemplate->table($GLOBALS['bx_profiler']->_aModules)
        );
    }

    function _formatTime ($i, $iPrecision = 3) {
        return round($i, $iPrecision) . ' sec';
    }

    function _formatBytes ($i) {
        if ($i > 1024*1024)
            return round($i/1024/1024, 1) . 'M';
        elseif ($i > 1024)
            return round($i/1024, 1) . 'K';        
        else
            return $i . 'B';
    }    

    function _isProfilerDisabled() {
        if (isset($GLOBALS['bx_profiler_disable']) || isset($_GET['bx_profiler_disable']))
            return true;
        if (
            preg_match('/get_rss_feed\.php/', $_SERVER['PHP_SELF']) || 
            preg_match('/fields\.parse\.php/', $_SERVER['PHP_SELF']) || 
			preg_match('/flash/', $_SERVER['PHP_SELF']) || 
			preg_match('/forum/', $_SERVER['PHP_SELF'])
		)
            return true;
        return false;
    }

    function _calcTime ($begin) {
        $i1 = explode(' ', microtime ());
        $i2 = explode(' ', $begin);
        return ($i1[0]+$i1[1]) - ($i2[0]+$i2[1]);
    }

    function _debugPrintArray ($mixed) {
        $sArgs .= 'Array(';
        foreach ($mixed as $mixed2)
            $sArgs .= (is_object($mixed2) ? $this->_debugPrintObject($mixed2) : $this->_debugPrintAny($mixed2)) . ',';
        $sArgs = substr($sArgs, 0, -1);
        $sArgs .= ')';        
        return $sArgs;
    }

    function _debugPrintObject ($mixed) {
        return get_class($mixed) . ' instance';
    }

    function _debugPrintAny ($mixed) {
        if (is_string)
            return "'" . (strlen($mixed) > $this->_sLogMaxArgLength ? substr($mixed, 0, $this->_sLogMaxArgLength) . '...' : $mixed) . "'";
        else
            return $mixed;
    }

    function _debugBackTrace ($iShifts = 0) {
        
        $a = debug_backtrace();
        while (--$iShifts > -1)
            array_shift($a);

        $s = '';
        foreach ($a as $r) {

            $sArgs = '';
            foreach ($r['args'] as $mixed) {
                switch (true) {
                case is_array($mixed):
                    $sArgs .= $this->_debugPrintArray($mixed);
                    break;
                case is_object($mixed):
                    $sArgs .= $this->_debugPrintObject($mixed);
                    break;
                default:
                    $sArgs .= $this->_debugPrintAny($mixed);
                }
                $sArgs .= ',';
            }
            $sArgs = substr($sArgs, 0, -1);
            
            $s .= "--------------------------------------\n";
            $s .= "{$r['line']} {$r['file']}\n";
            $s .= "{$r['class']}{$r['type']}{$r['function']} ({$sArgs});\n";
        }
        return $s;
    }    
}

if (BX_PROFILER) {
    $GLOBALS['bx_profiler'] = new BxProfiler($GLOBALS['bx_profiler_start']);
    if (!$GLOBALS['bx_profiler']->_isProfilerDisabled())
        register_shutdown_function (array ($GLOBALS['bx_profiler'], 'output'));
}

?>
